@@ -11,12 +11,13 @@
 #' @param height Height of the htmlwidget
 #'
 #' @export
+#' @keywords internal
 diffviewer_widget <- function(old, new, width = NULL, height = NULL,
   pattern = NULL)
 {
 
   if (xor(assertthat::is.dir(old), assertthat::is.dir(new))) {
-      stop("`old` and `new` must both be directories, or character vectors of filenames.")
+      abort("`old` and `new` must both be directories, or character vectors of filenames.")
   }
 
   # If `old` or `new` are directories, get a list of filenames from both directories
@@ -81,6 +82,7 @@ diffviewer_widget <- function(old, new, width = NULL, height = NULL,
 #' @param testname Name of test to compare.
 #'
 #' @export
+#' @keywords internal
 viewTestDiffWidget <- function(appDir = ".", testname = NULL, suffix = NULL) {
   testDir <- findTestsDir(appDir, quiet = TRUE)
   expected <- file.path(testDir, paste0(testname, "-expected", normalize_suffix(suffix)))
@@ -96,24 +98,25 @@ viewTestDiffWidget <- function(appDir = ".", testname = NULL, suffix = NULL) {
 #'   compare all test results for which there are differences.
 #' @param interactive If TRUE, use the interactive diff viewer, which runs in a
 #'   Shiny app. If FALSE, print a textual diff, generated by
-#'   \code{\link{textTestDiff}}.
-#' @param images Compare screenshot images (only used when \code{interactive} is
+#'   [textTestDiff()].
+#' @param images Compare screenshot images (only used when `interactive` is
 #'   FALSE).
 #' @param suffix An optional suffix for the expected results directory. For
-#'   example, if the suffix is \code{"mac"}, the expected directory would be
-#'   \code{mytest-expected-mac}.
+#'   example, if the suffix is `"mac"`, the expected directory would be
+#'   `mytest-expected-mac`.
 #'
-#' @return A character vector the same length as \code{testnames}, with
-#'   \code{"accept"} or \code{"reject"} for each test.
+#' @return A character vector the same length as `testnames`, with
+#'   `"accept"` or `"reject"` for each test.
 #'
-#' @seealso \code{\link{textTestDiff}} to get a text diff as a string.
+#' @seealso [textTestDiff()] to get a text diff as a string.
 #'
 #' @import shiny
 #' @export
+#' @keywords internal
 viewTestDiff <- function(
   appDir = ".",
   testnames = NULL,
-  interactive = base::interactive(),
+  interactive = is_interactive(),
   images = TRUE,
   suffix = NULL
 ) {
@@ -124,10 +127,10 @@ viewTestDiff <- function(
       testnames <- all_testnames(testDir, "-current")
     }
 
-    message("Differences in current results found for: ", paste(testnames, collapse = " "))
+    inform(c("Differences in current results found for: ", testnames))
 
     results <- lapply(testnames, function(testname) {
-      message("Viewing diff for ", testname)
+      inform(paste0("Viewing diff for ", testname))
       viewTestDiffSingle(appDir, testname, suffix)
     })
 
@@ -164,8 +167,9 @@ viewTestDiffSingle <- function(appDir = ".", testname = NULL, suffix = NULL) {
 #'
 #' @inheritParams viewTestDiff
 #' @param images Compare screenshot images.
-#' @seealso \code{\link{viewTestDiff}} for interactive diff viewer.
+#' @seealso [viewTestDiff()] for interactive diff viewer.
 #' @export
+#' @keywords internal
 textTestDiff <- function(
   appDir = ".",
   testnames = NULL,
